
import bpy
from mathutils import Color



#=====================================================
#                  SPINE
#=====================================================




# Add Rigify Samples => Spine
class OBJECT_OT_human_metarig_simple_spine_ra(bpy.types.Operator):
    '''Add Rigify Samples => Spine'''
    bl_idname = "object.human_metarig_simple_spine_ra"
    bl_label = "Add Spine"
    bl_options = {'REGISTER', 'UNDO'}

 

    def execute(self, context):
        

        # Check Rigify in preferences

        disable_Rigify = bpy.ops.preferences.addon_disable(module="rigify")

        if disable_Rigify:
            bpy.ops.preferences.addon_enable(module="rigify")

        else:
            pass
                    
  
        if bpy.context.mode == 'OBJECT':
            
            # Create single bone / go to Edot Mode / select and delete bone
            bpy.ops.object.armature_add(enter_editmode=0, align='WORLD', location=(0, 0, 0), scale=(1, 1, 1)) 
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)
            bpy.ops.armature.select_all(action='SELECT')
            bpy.ops.armature.delete()    
         
             
        else:
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)



        # Active object
        ob = bpy.context.object

        # Rig Name Variable
        if ob.type == 'ARMATURE':
            ob.name = 'metarig'
            ob.data.name = 'metarig'

        # CREATE SAMPLE

        arm = bpy.data.objects[ob.name].data

        for i in range(6):
            arm.rigify_colors.add()

        arm.rigify_colors[0].name = "Root"
        arm.rigify_colors[0].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[0].normal = Color((0.4353, 0.1843, 0.4157))
        arm.rigify_colors[0].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[0].standard_colors_lock = True
        arm.rigify_colors[1].name = "IK"
        arm.rigify_colors[1].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[1].normal = Color((0.6039, 0.0000, 0.0000))
        arm.rigify_colors[1].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[1].standard_colors_lock = True
        arm.rigify_colors[2].name = "Special"
        arm.rigify_colors[2].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[2].normal = Color((0.9569, 0.7882, 0.0471))
        arm.rigify_colors[2].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[2].standard_colors_lock = True
        arm.rigify_colors[3].name = "Tweak"
        arm.rigify_colors[3].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[3].normal = Color((0.0392, 0.2118, 0.5804))
        arm.rigify_colors[3].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[3].standard_colors_lock = True
        arm.rigify_colors[4].name = "FK"
        arm.rigify_colors[4].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[4].normal = Color((0.1176, 0.5686, 0.0353))
        arm.rigify_colors[4].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[4].standard_colors_lock = True
        arm.rigify_colors[5].name = "Extra"
        arm.rigify_colors[5].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[5].normal = Color((0.9686, 0.2510, 0.0941))
        arm.rigify_colors[5].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[5].standard_colors_lock = True

        bone_collections = {}

        for bcoll in list(arm.collections):
            arm.collections.remove(bcoll)

        def add_bone_collection(name, *, ui_row=0, ui_title='', sel_set=False, color_set_id=0):
            new_bcoll = arm.collections.new(name)
            new_bcoll.rigify_ui_row = ui_row
            new_bcoll.rigify_ui_title = ui_title
            new_bcoll.rigify_sel_set = sel_set
            new_bcoll.rigify_color_set_id = color_set_id
            bone_collections[name] = new_bcoll

        def assign_bone_collections(pose_bone, *coll_names):
            assert not len(pose_bone.bone.collections)
            for name in coll_names:
                bone_collections[name].assign(pose_bone)

        def assign_bone_collection_refs(params, attr_name, *coll_names):
            ref_list = getattr(params, attr_name + '_coll_refs', None)
            if ref_list is not None:
                for name in coll_names:
                    ref_list.add().set_collection(bone_collections[name])

        add_bone_collection('Face', ui_row=1, color_set_id=5)
        add_bone_collection('Face (Primary)', ui_row=2, ui_title='(Primary)', color_set_id=2)
        add_bone_collection('Face (Secondary)', ui_row=2, ui_title='(Secondary)', color_set_id=3)
        add_bone_collection('Torso', ui_row=4, color_set_id=3)
        add_bone_collection('Torso (Tweak)', ui_row=5, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Fingers', ui_row=7, color_set_id=6)
        add_bone_collection('Fingers (Detail)', ui_row=8, ui_title='(Detail)', color_set_id=5)
        add_bone_collection('Arm.L (IK)', ui_row=10, color_set_id=2)
        add_bone_collection('Arm.L (FK)', ui_row=11, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Arm.L (Tweak)', ui_row=12, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Arm.R (IK)', ui_row=10, color_set_id=2)
        add_bone_collection('Arm.R (FK)', ui_row=11, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Arm.R (Tweak)', ui_row=12, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Leg.L (IK)', ui_row=14, color_set_id=2)
        add_bone_collection('Leg.L (FK)', ui_row=15, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Leg.L (Tweak)', ui_row=16, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Leg.R (IK)', ui_row=14, color_set_id=2)
        add_bone_collection('Leg.R (FK)', ui_row=15, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Leg.R (Tweak)', ui_row=16, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Root', ui_row=19, color_set_id=1)

        bones = {}

        bone = arm.edit_bones.new('spine')
        bone.head = 0.0000, 0.0552, 1.0099
        bone.tail = 0.0000, 0.0172, 1.1573
        bone.roll = 0.0000
        bone.use_connect = False
        bones['spine'] = bone.name
        bone = arm.edit_bones.new('spine.001')
        bone.head = 0.0000, 0.0172, 1.1573
        bone.tail = 0.0000, 0.0004, 1.2929
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine']]
        bones['spine.001'] = bone.name
        bone = arm.edit_bones.new('spine.002')
        bone.head = 0.0000, 0.0004, 1.2929
        bone.tail = 0.0000, 0.0059, 1.4657
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.001']]
        bones['spine.002'] = bone.name
        bone = arm.edit_bones.new('spine.003')
        bone.head = 0.0000, 0.0059, 1.4657
        bone.tail = 0.0000, 0.0114, 1.6582
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.002']]
        bones['spine.003'] = bone.name

        obj = bpy.context.object 

        bpy.ops.object.mode_set(mode='OBJECT')
        pbone = obj.pose.bones[bones['spine']]
        pbone.rigify_type = 'spines.basic_spine'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Torso (Tweak)')
        assign_bone_collection_refs(pbone.rigify_parameters, 'fk', 'Torso (Tweak)')
        pbone = obj.pose.bones[bones['spine.001']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        pbone = obj.pose.bones[bones['spine.002']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        pbone = obj.pose.bones[bones['spine.003']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')

        bpy.ops.object.mode_set(mode='EDIT')
        for bone in arm.edit_bones:
            bone.select = False
            bone.select_head = False
            bone.select_tail = False
        for b in bones:
            bone = arm.edit_bones[bones[b]]
            bone.select = True
            bone.select_head = True
            bone.select_tail = True
            bone.bbone_x = bone.bbone_z = bone.length * 0.05
            arm.edit_bones.active = bone

        arm.collections.active_index = 0


        # View - Frame Selected
        bpy.ops.view3d.view_selected(use_all_regions=0)


        return {'FINISHED'}  




#=====================================================
#        END      SPINE
#=====================================================



#=====================================================
#                  SPINE AND HEAD
#=====================================================



# Add Rigify Samples => Spine with Head
class OBJECT_OT_human_metarig_simple_spine_with_head_ra(bpy.types.Operator):
    '''Add Rigify Samples => Spine with Head'''
    bl_idname = "object.human_metarig_simple_spine_with_head_ra"
    bl_label = "Add Spine / Head"
    bl_options = {'REGISTER', 'UNDO'}
 
 
 
    def execute(self, context):
        

        # Check Rigify in preferences

        disable_Rigify = bpy.ops.preferences.addon_disable(module="rigify")

        if disable_Rigify:
            bpy.ops.preferences.addon_enable(module="rigify")

        else:
            pass
                    
  
        if bpy.context.mode == 'OBJECT':
            
            # Create single bone / go to Edot Mode / select and delete bone
            bpy.ops.object.armature_add(enter_editmode=0, align='WORLD', location=(0, 0, 0), scale=(1, 1, 1)) 
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)
            bpy.ops.armature.select_all(action='SELECT')
            bpy.ops.armature.delete()    
         
             
        else:
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)



        # Active object
        ob = bpy.context.object

        # Rig Name Variable
        if ob.type == 'ARMATURE':
            ob.name = 'metarig'
            ob.data.name = 'metarig'

        # CREATE SAMPLE

        arm = bpy.data.objects[ob.name].data

        for i in range(6):
            arm.rigify_colors.add()

        arm.rigify_colors[0].name = "Root"
        arm.rigify_colors[0].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[0].normal = Color((0.4353, 0.1843, 0.4157))
        arm.rigify_colors[0].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[0].standard_colors_lock = True
        arm.rigify_colors[1].name = "IK"
        arm.rigify_colors[1].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[1].normal = Color((0.6039, 0.0000, 0.0000))
        arm.rigify_colors[1].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[1].standard_colors_lock = True
        arm.rigify_colors[2].name = "Special"
        arm.rigify_colors[2].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[2].normal = Color((0.9569, 0.7882, 0.0471))
        arm.rigify_colors[2].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[2].standard_colors_lock = True
        arm.rigify_colors[3].name = "Tweak"
        arm.rigify_colors[3].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[3].normal = Color((0.0392, 0.2118, 0.5804))
        arm.rigify_colors[3].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[3].standard_colors_lock = True
        arm.rigify_colors[4].name = "FK"
        arm.rigify_colors[4].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[4].normal = Color((0.1176, 0.5686, 0.0353))
        arm.rigify_colors[4].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[4].standard_colors_lock = True
        arm.rigify_colors[5].name = "Extra"
        arm.rigify_colors[5].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[5].normal = Color((0.9686, 0.2510, 0.0941))
        arm.rigify_colors[5].select = Color((0.3137, 0.7843, 1.0000))
        arm.rigify_colors[5].standard_colors_lock = True

        bone_collections = {}

        for bcoll in list(arm.collections):
            arm.collections.remove(bcoll)

        def add_bone_collection(name, *, ui_row=0, ui_title='', sel_set=False, color_set_id=0):
            new_bcoll = arm.collections.new(name)
            new_bcoll.rigify_ui_row = ui_row
            new_bcoll.rigify_ui_title = ui_title
            new_bcoll.rigify_sel_set = sel_set
            new_bcoll.rigify_color_set_id = color_set_id
            bone_collections[name] = new_bcoll

        def assign_bone_collections(pose_bone, *coll_names):
            assert not len(pose_bone.bone.collections)
            for name in coll_names:
                bone_collections[name].assign(pose_bone)

        def assign_bone_collection_refs(params, attr_name, *coll_names):
            ref_list = getattr(params, attr_name + '_coll_refs', None)
            if ref_list is not None:
                for name in coll_names:
                    ref_list.add().set_collection(bone_collections[name])

        add_bone_collection('Face', ui_row=1, color_set_id=5)
        add_bone_collection('Face (Primary)', ui_row=2, ui_title='(Primary)', color_set_id=2)
        add_bone_collection('Face (Secondary)', ui_row=2, ui_title='(Secondary)', color_set_id=3)
        add_bone_collection('Torso', ui_row=4, color_set_id=3)
        add_bone_collection('Torso (Tweak)', ui_row=5, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Fingers', ui_row=7, color_set_id=6)
        add_bone_collection('Fingers (Detail)', ui_row=8, ui_title='(Detail)', color_set_id=5)
        add_bone_collection('Arm.L (IK)', ui_row=10, color_set_id=2)
        add_bone_collection('Arm.L (FK)', ui_row=11, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Arm.L (Tweak)', ui_row=12, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Arm.R (IK)', ui_row=10, color_set_id=2)
        add_bone_collection('Arm.R (FK)', ui_row=11, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Arm.R (Tweak)', ui_row=12, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Leg.L (IK)', ui_row=14, color_set_id=2)
        add_bone_collection('Leg.L (FK)', ui_row=15, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Leg.L (Tweak)', ui_row=16, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Leg.R (IK)', ui_row=14, color_set_id=2)
        add_bone_collection('Leg.R (FK)', ui_row=15, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Leg.R (Tweak)', ui_row=16, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Root', ui_row=19, color_set_id=1)

        bones = {}

        bone = arm.edit_bones.new('spine')
        bone.head = 0.0000, 0.0552, 1.0099
        bone.tail = 0.0000, 0.0172, 1.1573
        bone.roll = 0.0000
        bone.use_connect = False
        bones['spine'] = bone.name
        bone = arm.edit_bones.new('spine.001')
        bone.head = 0.0000, 0.0172, 1.1573
        bone.tail = 0.0000, 0.0004, 1.2929
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine']]
        bones['spine.001'] = bone.name
        bone = arm.edit_bones.new('spine.002')
        bone.head = 0.0000, 0.0004, 1.2929
        bone.tail = 0.0000, 0.0059, 1.4657
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.001']]
        bones['spine.002'] = bone.name
        bone = arm.edit_bones.new('spine.003')
        bone.head = 0.0000, 0.0059, 1.4657
        bone.tail = 0.0000, 0.0114, 1.6582
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.002']]
        bones['spine.003'] = bone.name
        bone = arm.edit_bones.new('spine.004')
        bone.head = 0.0000, 0.0114, 1.6582
        bone.tail = 0.0000, -0.0130, 1.7197
        bone.roll = 0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.003']]
        bones['spine.004'] = bone.name
        bone = arm.edit_bones.new('spine.005')
        bone.head = 0.0000, -0.0130, 1.7197
        bone.tail = 0.0000, -0.0247, 1.7813
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.004']]
        bones['spine.005'] = bone.name
        bone = arm.edit_bones.new('spine.006')
        bone.head = 0.0000, -0.0247, 1.7813
        bone.tail = 0.0000, -0.0247, 1.9796
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.005']]
        bones['spine.006'] = bone.name
        
        obj = bpy.context.object         

        bpy.ops.object.mode_set(mode='OBJECT')
        pbone = obj.pose.bones[bones['spine']]
        pbone.rigify_type = 'spines.basic_spine'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Torso (Tweak)')
        assign_bone_collection_refs(pbone.rigify_parameters, 'fk', 'Torso (Tweak)')
        pbone = obj.pose.bones[bones['spine.001']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        pbone = obj.pose.bones[bones['spine.002']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        pbone = obj.pose.bones[bones['spine.003']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        pbone = obj.pose.bones[bones['spine.004']]
        pbone.rigify_type = 'spines.super_head'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        try:
            pbone.rigify_parameters.connect_chain = True
        except AttributeError:
            pass
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Torso (Tweak)')
        pbone = obj.pose.bones[bones['spine.005']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')
        pbone = obj.pose.bones[bones['spine.006']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Torso')

        bpy.ops.object.mode_set(mode='EDIT')
        for bone in arm.edit_bones:
            bone.select = False
            bone.select_head = False
            bone.select_tail = False
        for b in bones:
            bone = arm.edit_bones[bones[b]]
            bone.select = True
            bone.select_head = True
            bone.select_tail = True
            bone.bbone_x = bone.bbone_z = bone.length * 0.05
            arm.edit_bones.active = bone

        arm.collections.active_index = 0
        
        

        bpy.context.scene.tool_settings.transform_pivot_point = 'ACTIVE_ELEMENT'
          
        # View - Frame Selected
        bpy.ops.view3d.view_selected(use_all_regions=0)


        return {'FINISHED'}  




#=====================================================
#              END    SPINE AND HEAD
#=====================================================






classes = [ OBJECT_OT_human_metarig_simple_spine_ra,
            OBJECT_OT_human_metarig_simple_spine_with_head_ra,

          ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)



def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)




if __name__ == "__main__":
    register()







