import bpy
from mathutils import Color



#=====================================================
#            RIGIFY SAMPLES 
#=====================================================
 

#=====================================================
#            RIGIFY ZOO METARIG - HORSE   
#=====================================================
 
 
 

# Add Rigify Metarig - Horse
class OBJECT_OT_rigify_zoo_horse_metarig_ra(bpy.types.Operator):
    '''Rigify ZOO Metarig - Horse'''
    bl_idname = "object.rigify_zoo_horse_metarig_ra"
    bl_label = "Horse Metarig"
    bl_options = {'REGISTER', 'UNDO'}

 
    def execute(self, context):
        

        # Check Rigify in preferences

        disable_Rigify = bpy.ops.preferences.addon_disable(module="rigify")

        if disable_Rigify:
            bpy.ops.preferences.addon_enable(module="rigify")

        else:
            pass
                    
        
  
        if bpy.context.mode == 'OBJECT':
            
            # Create single bone / go to Edot Mode / select and delete bone
            bpy.ops.object.armature_add(enter_editmode=0, align='WORLD', location=(0, 0, 0), scale=(1, 1, 1)) 
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)
            bpy.ops.armature.select_all(action='SELECT')
            bpy.ops.armature.delete()    
         
             
        else:
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)



        # Active object
        ob = bpy.context.object

        # Rig Name Variable
        if ob.type == 'ARMATURE':
            ob.name = ob.name

        # CREATE SAMPLE


        arm = bpy.data.objects[ob.name].data


        for i in range(6):
            arm.rigify_colors.add()

        arm.rigify_colors[0].name = "Root"
        arm.rigify_colors[0].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[0].normal = Color((0.4353, 0.1843, 0.4157))
        arm.rigify_colors[0].select = Color((0.3140, 0.7840, 1.0000))
        arm.rigify_colors[0].standard_colors_lock = True
        arm.rigify_colors[1].name = "IK"
        arm.rigify_colors[1].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[1].normal = Color((0.6039, 0.0000, 0.0000))
        arm.rigify_colors[1].select = Color((0.3140, 0.7840, 1.0000))
        arm.rigify_colors[1].standard_colors_lock = True
        arm.rigify_colors[2].name = "Special"
        arm.rigify_colors[2].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[2].normal = Color((0.9569, 0.7882, 0.0471))
        arm.rigify_colors[2].select = Color((0.3140, 0.7840, 1.0000))
        arm.rigify_colors[2].standard_colors_lock = True
        arm.rigify_colors[3].name = "Tweak"
        arm.rigify_colors[3].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[3].normal = Color((0.0392, 0.2118, 0.5804))
        arm.rigify_colors[3].select = Color((0.3140, 0.7840, 1.0000))
        arm.rigify_colors[3].standard_colors_lock = True
        arm.rigify_colors[4].name = "FK"
        arm.rigify_colors[4].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[4].normal = Color((0.1176, 0.5686, 0.0353))
        arm.rigify_colors[4].select = Color((0.3140, 0.7840, 1.0000))
        arm.rigify_colors[4].standard_colors_lock = True
        arm.rigify_colors[5].name = "Extra"
        arm.rigify_colors[5].active = Color((0.5490, 1.0000, 1.0000))
        arm.rigify_colors[5].normal = Color((0.9686, 0.2510, 0.0941))
        arm.rigify_colors[5].select = Color((0.3140, 0.7840, 1.0000))
        arm.rigify_colors[5].standard_colors_lock = True

        bone_collections = {}

        for bcoll in list(arm.collections):
            arm.collections.remove(bcoll)

        def add_bone_collection(name, *, ui_row=0, ui_title='', sel_set=False, color_set_id=0):
            new_bcoll = arm.collections.new(name)
            new_bcoll.rigify_ui_row = ui_row
            new_bcoll.rigify_ui_title = ui_title
            new_bcoll.rigify_sel_set = sel_set
            new_bcoll.rigify_color_set_id = color_set_id
            bone_collections[name] = new_bcoll

        def assign_bone_collections(pose_bone, *coll_names):
            assert not len(pose_bone.bone.collections)
            for name in coll_names:
                bone_collections[name].assign(pose_bone)

        def assign_bone_collection_refs(params, attr_name, *coll_names):
            ref_list = getattr(params, attr_name + '_coll_refs', None)
            if ref_list is not None:
                for name in coll_names:
                    ref_list.add().set_collection(bone_collections[name])

        add_bone_collection('Face', ui_row=1, color_set_id=5)
        add_bone_collection('Face (Primary)', ui_row=2, ui_title='(Primary)', color_set_id=4)
        add_bone_collection('Spine', ui_row=4, color_set_id=3)
        add_bone_collection('Spine (Tweak)', ui_row=5, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Arm.L (IK)', ui_row=7, color_set_id=2)
        add_bone_collection('Arm.L (FK)', ui_row=8, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Arm.L (Tweak)', ui_row=9, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Arm.R (IK)', ui_row=7, color_set_id=2)
        add_bone_collection('Arm.R (FK)', ui_row=8, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Arm.R (Tweak)', ui_row=9, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Leg.L (IK)', ui_row=11, color_set_id=2)
        add_bone_collection('Leg.L (FK)', ui_row=12, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Leg.L (Tweak)', ui_row=13, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Leg.R (IK)', ui_row=11, color_set_id=2)
        add_bone_collection('Leg.R (FK)', ui_row=12, ui_title='(FK)', color_set_id=5)
        add_bone_collection('Leg.R (Tweak)', ui_row=13, ui_title='(Tweak)', color_set_id=4)
        add_bone_collection('Tail', ui_row=15, color_set_id=6)
        add_bone_collection('Hair', ui_row=2, color_set_id=6)
        add_bone_collection('Root', ui_row=18, color_set_id=1)

        bones = {}

        bone = arm.edit_bones.new('spine.001')
        bone.head = 0.0000, 0.5403, 1.1974
        bone.tail = 0.0000, 0.3066, 1.2815
        bone.roll = -0.0000
        bone.use_connect = False
        bones['spine.001'] = bone.name
        bone = arm.edit_bones.new('tail.001')
        bone.head = 0.0000, 0.5947, 1.3219
        bone.tail = 0.0000, 0.7229, 1.2657
        bone.roll = 0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.001']]
        bones['tail.001'] = bone.name
        bone = arm.edit_bones.new('spine.002')
        bone.head = 0.0000, 0.3066, 1.2815
        bone.tail = 0.0000, 0.1602, 1.2573
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.001']]
        bones['spine.002'] = bone.name
        bone = arm.edit_bones.new('tail.002')
        bone.head = 0.0000, 0.7229, 1.2657
        bone.tail = 0.0000, 0.8126, 1.1582
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['tail.001']]
        bones['tail.002'] = bone.name
        bone = arm.edit_bones.new('spine.003')
        bone.head = 0.0000, 0.1602, 1.2573
        bone.tail = 0.0000, -0.0226, 1.2225
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.002']]
        bones['spine.003'] = bone.name
        bone = arm.edit_bones.new('pelvis.L')
        bone.head = 0.0000, 0.5403, 1.1974
        bone.tail = -0.0980, 0.3514, 1.3329
        bone.roll = -0.3258
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.002']]
        bones['pelvis.L'] = bone.name
        bone = arm.edit_bones.new('pelvis.R')
        bone.head = 0.0000, 0.5403, 1.1974
        bone.tail = 0.0980, 0.3514, 1.3329
        bone.roll = 0.3258
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.002']]
        bones['pelvis.R'] = bone.name
        bone = arm.edit_bones.new('hip')
        bone.head = 0.0000, 0.5403, 1.1974
        bone.tail = 0.0000, 0.4636, 1.0212
        bone.roll = 0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.002']]
        bones['hip'] = bone.name
        bone = arm.edit_bones.new('tail.003')
        bone.head = 0.0000, 0.8126, 1.1582
        bone.tail = 0.0000, 0.8662, 1.0289
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['tail.002']]
        bones['tail.003'] = bone.name
        bone = arm.edit_bones.new('spine.004')
        bone.head = 0.0000, -0.0226, 1.2225
        bone.tail = 0.0000, -0.2976, 1.2194
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.003']]
        bones['spine.004'] = bone.name
        bone = arm.edit_bones.new('abdomen')
        bone.head = 0.0000, 0.1503, 1.1819
        bone.tail = 0.0000, 0.1869, 0.9629
        bone.roll = 0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.003']]
        bones['abdomen'] = bone.name
        bone = arm.edit_bones.new('thigh.L')
        bone.head = 0.1605, 0.4663, 1.1109
        bone.tail = 0.1616, 0.4541, 0.8213
        bone.roll = 3.1369
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['pelvis.L']]
        bones['thigh.L'] = bone.name
        bone = arm.edit_bones.new('thigh.R')
        bone.head = -0.1605, 0.4663, 1.1109
        bone.tail = -0.1616, 0.4541, 0.8213
        bone.roll = -3.1369
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['pelvis.R']]
        bones['thigh.R'] = bone.name
        bone = arm.edit_bones.new('tail.004')
        bone.head = 0.0000, 0.8662, 1.0289
        bone.tail = 0.0000, 0.8949, 0.8919
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['tail.003']]
        bones['tail.004'] = bone.name
        bone = arm.edit_bones.new('spine.005')
        bone.head = 0.0000, -0.2976, 1.2194
        bone.tail = 0.0000, -0.4757, 1.2471
        bone.roll = 0.0001
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.004']]
        bones['spine.005'] = bone.name
        bone = arm.edit_bones.new('lower_leg.L')
        bone.head = 0.1616, 0.4541, 0.8213
        bone.tail = 0.1616, 0.5796, 0.5283
        bone.roll = 3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['thigh.L']]
        bones['lower_leg.L'] = bone.name
        bone = arm.edit_bones.new('lower_leg.R')
        bone.head = -0.1616, 0.4541, 0.8213
        bone.tail = -0.1616, 0.5796, 0.5283
        bone.roll = -3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['thigh.R']]
        bones['lower_leg.R'] = bone.name
        bone = arm.edit_bones.new('tail.005')
        bone.head = 0.0000, 0.8949, 0.8919
        bone.tail = 0.0000, 0.9309, 0.7565
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['tail.004']]
        bones['tail.005'] = bone.name
        bone = arm.edit_bones.new('spine.006')
        bone.head = 0.0000, -0.4757, 1.2471
        bone.tail = 0.0000, -0.5908, 1.2785
        bone.roll = 0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['spine.005']]
        bones['spine.006'] = bone.name
        bone = arm.edit_bones.new('shoulder.L')
        bone.head = 0.0822, -0.4983, 1.4009
        bone.tail = 0.1346, -0.6484, 1.0838
        bone.roll = -0.1890
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.005']]
        bones['shoulder.L'] = bone.name
        bone = arm.edit_bones.new('breast.L')
        bone.head = -0.0719, -0.4916, 1.1731
        bone.tail = -0.0719, -0.6919, 0.9810
        bone.roll = 0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.005']]
        bones['breast.L'] = bone.name
        bone = arm.edit_bones.new('shoulder.R')
        bone.head = -0.0822, -0.4983, 1.4009
        bone.tail = -0.1346, -0.6484, 1.0838
        bone.roll = 0.1890
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.005']]
        bones['shoulder.R'] = bone.name
        bone = arm.edit_bones.new('breast.R')
        bone.head = 0.0719, -0.4916, 1.1731
        bone.tail = 0.0719, -0.6919, 0.9810
        bone.roll = 0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.005']]
        bones['breast.R'] = bone.name
        bone = arm.edit_bones.new('chest')
        bone.head = 0.0000, -0.2180, 1.2173
        bone.tail = 0.0000, -0.2239, 0.8605
        bone.roll = 0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.005']]
        bones['chest'] = bone.name
        bone = arm.edit_bones.new('hind_foot.L')
        bone.head = 0.1616, 0.5796, 0.5283
        bone.tail = 0.1616, 0.6062, 0.1871
        bone.roll = -3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['lower_leg.L']]
        bones['hind_foot.L'] = bone.name
        bone = arm.edit_bones.new('hind_foot.R')
        bone.head = -0.1616, 0.5796, 0.5283
        bone.tail = -0.1616, 0.6062, 0.1871
        bone.roll = 3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['lower_leg.R']]
        bones['hind_foot.R'] = bone.name
        bone = arm.edit_bones.new('neck.001')
        bone.head = 0.0000, -0.5908, 1.2785
        bone.tail = 0.0000, -0.6699, 1.3504
        bone.roll = -0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['spine.006']]
        bones['neck.001'] = bone.name
        bone = arm.edit_bones.new('upper_arm.L')
        bone.head = 0.1539, -0.6429, 1.0469
        bone.tail = 0.1539, -0.5644, 0.8280
        bone.roll = -3.1416
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['shoulder.L']]
        bones['upper_arm.L'] = bone.name
        bone = arm.edit_bones.new('upper_arm.R')
        bone.head = -0.1539, -0.6429, 1.0469
        bone.tail = -0.1539, -0.5644, 0.8280
        bone.roll = 3.1416
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['shoulder.R']]
        bones['upper_arm.R'] = bone.name
        bone = arm.edit_bones.new('r_toe.L')
        bone.head = 0.1616, 0.6062, 0.1871
        bone.tail = 0.1616, 0.5767, 0.0861
        bone.roll = -3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['hind_foot.L']]
        bones['r_toe.L'] = bone.name
        bone = arm.edit_bones.new('r_toe.R')
        bone.head = -0.1616, 0.6062, 0.1871
        bone.tail = -0.1616, 0.5767, 0.0861
        bone.roll = 3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['hind_foot.R']]
        bones['r_toe.R'] = bone.name
        bone = arm.edit_bones.new('neck.002')
        bone.head = 0.0000, -0.6699, 1.3504
        bone.tail = 0.0000, -0.7487, 1.4479
        bone.roll = -0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['neck.001']]
        bones['neck.002'] = bone.name
        bone = arm.edit_bones.new('forearm.L')
        bone.head = 0.1539, -0.5644, 0.8280
        bone.tail = 0.1539, -0.5499, 0.4490
        bone.roll = 3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['upper_arm.L']]
        bones['forearm.L'] = bone.name
        bone = arm.edit_bones.new('forearm.R')
        bone.head = -0.1539, -0.5644, 0.8280
        bone.tail = -0.1539, -0.5499, 0.4490
        bone.roll = -3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['upper_arm.R']]
        bones['forearm.R'] = bone.name
        bone = arm.edit_bones.new('r_hoof.L')
        bone.head = 0.1616, 0.5767, 0.0861
        bone.tail = 0.1616, 0.5170, 0.0007
        bone.roll = -3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['r_toe.L']]
        bones['r_hoof.L'] = bone.name
        bone = arm.edit_bones.new('r_hoof.R')
        bone.head = -0.1616, 0.5767, 0.0861
        bone.tail = -0.1616, 0.5170, 0.0007
        bone.roll = 3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['r_toe.R']]
        bones['r_hoof.R'] = bone.name
        bone = arm.edit_bones.new('neck.003')
        bone.head = 0.0000, -0.7487, 1.4479
        bone.tail = 0.0000, -0.8447, 1.5479
        bone.roll = -0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['neck.002']]
        bones['neck.003'] = bone.name
        bone = arm.edit_bones.new('forefoot.L')
        bone.head = 0.1539, -0.5499, 0.4490
        bone.tail = 0.1539, -0.5221, 0.1579
        bone.roll = -3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['forearm.L']]
        bones['forefoot.L'] = bone.name
        bone = arm.edit_bones.new('forefoot.R')
        bone.head = -0.1539, -0.5499, 0.4490
        bone.tail = -0.1539, -0.5221, 0.1579
        bone.roll = 3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['forearm.R']]
        bones['forefoot.R'] = bone.name
        bone = arm.edit_bones.new('neck.004')
        bone.head = 0.0000, -0.8447, 1.5479
        bone.tail = 0.0000, -0.9814, 1.6223
        bone.roll = -0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['neck.003']]
        bones['neck.004'] = bone.name
        bone = arm.edit_bones.new('f_toe.L')
        bone.head = 0.1539, -0.5221, 0.1579
        bone.tail = 0.1539, -0.5716, 0.0672
        bone.roll = 3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['forefoot.L']]
        bones['f_toe.L'] = bone.name
        bone = arm.edit_bones.new('f_toe.R')
        bone.head = -0.1539, -0.5221, 0.1579
        bone.tail = -0.1539, -0.5716, 0.0672
        bone.roll = -3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['forefoot.R']]
        bones['f_toe.R'] = bone.name
        bone = arm.edit_bones.new('head')
        bone.head = 0.0000, -0.9814, 1.6223
        bone.tail = 0.0000, -1.1679, 1.6885
        bone.roll = -0.0000
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['neck.004']]
        bones['head'] = bone.name
        bone = arm.edit_bones.new('f_hoof.L')
        bone.head = 0.1539, -0.5716, 0.0672
        bone.tail = 0.1539, -0.6251, 0.0007
        bone.roll = -3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['f_toe.L']]
        bones['f_hoof.L'] = bone.name
        bone = arm.edit_bones.new('f_hoof.R')
        bone.head = -0.1539, -0.5716, 0.0672
        bone.tail = -0.1539, -0.6251, 0.0007
        bone.roll = 3.1416
        bone.use_connect = True
        bone.parent = arm.edit_bones[bones['f_toe.R']]
        bones['f_hoof.R'] = bone.name
        bone = arm.edit_bones.new('skull')
        bone.head = 0.0000, -1.0585, 1.5732
        bone.tail = 0.0000, -1.3560, 1.3327
        bone.roll = -0.0000
        bone.use_connect = False
        bone.parent = arm.edit_bones[bones['head']]
        bones['skull'] = bone.name

        obj = bpy.context.object

        bpy.ops.object.mode_set(mode='OBJECT')
        pbone = obj.pose.bones[bones['spine.001']]
        pbone.rigify_type = 'spines.basic_spine'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        try:
            pbone.rigify_parameters.pivot_pos = 2
        except AttributeError:
            pass
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Spine (Tweak)')
        assign_bone_collection_refs(pbone.rigify_parameters, 'fk', 'Spine (Tweak)')
        pbone = obj.pose.bones[bones['tail.001']]
        pbone.rigify_type = 'spines.basic_tail'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Tail')
        try:
            pbone.rigify_parameters.copy_rotation_axes = (True, False, True)
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.connect_chain = False
        except AttributeError:
            pass
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Spine (Tweak)')
        pbone = obj.pose.bones[bones['spine.002']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['tail.002']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Tail')
        pbone = obj.pose.bones[bones['spine.003']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['pelvis.L']]
        pbone.rigify_type = 'basic.super_copy'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        try:
            pbone.rigify_parameters.make_widget = False
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.make_control = False
        except AttributeError:
            pass
        pbone = obj.pose.bones[bones['pelvis.R']]
        pbone.rigify_type = 'basic.super_copy'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        try:
            pbone.rigify_parameters.make_widget = False
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.make_control = False
        except AttributeError:
            pass
        pbone = obj.pose.bones[bones['hip']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['tail.003']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Tail')
        pbone = obj.pose.bones[bones['spine.004']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['abdomen']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine (Tweak)')
        pbone = obj.pose.bones[bones['thigh.L']]
        pbone.rigify_type = 'limbs.rear_paw'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.L (IK)')
        try:
            pbone.rigify_parameters.segments = 2
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.ik_local_location = False
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.bbones = 10
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.fk_layers_extra = True
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.tweak_layers_extra = True
        except AttributeError:
            pass
        assign_bone_collection_refs(pbone.rigify_parameters, 'fk', 'Leg.L (FK)')
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Leg.L (Tweak)')
        pbone = obj.pose.bones[bones['thigh.R']]
        pbone.rigify_type = 'limbs.rear_paw'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.R (IK)')
        try:
            pbone.rigify_parameters.segments = 2
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.ik_local_location = False
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.bbones = 10
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.fk_layers_extra = True
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.tweak_layers_extra = True
        except AttributeError:
            pass
        assign_bone_collection_refs(pbone.rigify_parameters, 'fk', 'Leg.R (FK)')
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Leg.R (Tweak)')
        pbone = obj.pose.bones[bones['tail.004']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Tail')
        pbone = obj.pose.bones[bones['spine.005']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['lower_leg.L']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.L (IK)')
        pbone = obj.pose.bones[bones['lower_leg.R']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.R (IK)')
        pbone = obj.pose.bones[bones['tail.005']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Tail')
        pbone = obj.pose.bones[bones['spine.006']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['shoulder.L']]
        pbone.rigify_type = 'basic.super_copy'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.L (IK)')
        try:
            pbone.rigify_parameters.make_widget = False
        except AttributeError:
            pass
        pbone = obj.pose.bones[bones['breast.L']]
        pbone.rigify_type = 'basic.super_copy'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine (Tweak)')
        pbone = obj.pose.bones[bones['shoulder.R']]
        pbone.rigify_type = 'basic.super_copy'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.R (IK)')
        try:
            pbone.rigify_parameters.make_widget = False
        except AttributeError:
            pass
        pbone = obj.pose.bones[bones['breast.R']]
        pbone.rigify_type = 'basic.super_copy'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine (Tweak)')
        pbone = obj.pose.bones[bones['chest']]
        pbone.rigify_type = 'basic.super_copy'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine (Tweak)')
        pbone = obj.pose.bones[bones['hind_foot.L']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.L (IK)')
        pbone = obj.pose.bones[bones['hind_foot.R']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.R (IK)')
        pbone = obj.pose.bones[bones['neck.001']]
        pbone.rigify_type = 'spines.super_head'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        try:
            pbone.rigify_parameters.connect_chain = True
        except AttributeError:
            pass
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Spine (Tweak)')
        pbone = obj.pose.bones[bones['upper_arm.L']]
        pbone.rigify_type = 'limbs.front_paw'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.L (IK)')
        try:
            pbone.rigify_parameters.limb_type = 'paw'
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.ik_local_location = False
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.front_paw_heel_influence = 0.6000000238418579
        except AttributeError:
            pass
        assign_bone_collection_refs(pbone.rigify_parameters, 'fk', 'Arm.L (FK)')
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Arm.L (Tweak)')
        pbone = obj.pose.bones[bones['upper_arm.R']]
        pbone.rigify_type = 'limbs.front_paw'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.R (IK)')
        try:
            pbone.rigify_parameters.limb_type = 'paw'
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.ik_local_location = False
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.front_paw_heel_influence = 0.6000000238418579
        except AttributeError:
            pass
        assign_bone_collection_refs(pbone.rigify_parameters, 'fk', 'Arm.R (FK)')
        assign_bone_collection_refs(pbone.rigify_parameters, 'tweak', 'Arm.R (Tweak)')
        pbone = obj.pose.bones[bones['r_toe.L']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.L (IK)')
        pbone = obj.pose.bones[bones['r_toe.R']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.R (IK)')
        pbone = obj.pose.bones[bones['neck.002']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['forearm.L']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.L (IK)')
        pbone = obj.pose.bones[bones['forearm.R']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.R (IK)')
        pbone = obj.pose.bones[bones['r_hoof.L']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.L (IK)')
        pbone = obj.pose.bones[bones['r_hoof.R']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Leg.R (IK)')
        pbone = obj.pose.bones[bones['neck.003']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['forefoot.L']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.L (IK)')
        pbone = obj.pose.bones[bones['forefoot.R']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.R (IK)')
        pbone = obj.pose.bones[bones['neck.004']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['f_toe.L']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.L (IK)')
        pbone = obj.pose.bones[bones['f_toe.R']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.R (IK)')
        pbone = obj.pose.bones[bones['head']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        pbone = obj.pose.bones[bones['f_hoof.L']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.L (IK)')
        pbone = obj.pose.bones[bones['f_hoof.R']]
        pbone.rigify_type = ''
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Arm.R (IK)')
        pbone = obj.pose.bones[bones['skull']]
        pbone.rigify_type = 'basic.super_copy'
        pbone.lock_location = (False, False, False)
        pbone.lock_rotation = (False, False, False)
        pbone.lock_rotation_w = False
        pbone.lock_scale = (False, False, False)
        pbone.rotation_mode = 'QUATERNION'
        assign_bone_collections(pbone, 'Spine')
        try:
            pbone.rigify_parameters.make_widget = False
        except AttributeError:
            pass
        try:
            pbone.rigify_parameters.make_control = False
        except AttributeError:
            pass

        bpy.ops.object.mode_set(mode='EDIT')
        for bone in arm.edit_bones:
            bone.select = False
            bone.select_head = False
            bone.select_tail = False
        for b in bones:
            bone = arm.edit_bones[bones[b]]
            bone.select = True
            bone.select_head = True
            bone.select_tail = True
            bone.bbone_x = bone.bbone_z = bone.length * 0.05
            arm.edit_bones.active = bone

        arm.collections.active_index = 0


 
        # View - Frame Selected
        bpy.ops.view3d.view_selected(use_all_regions=0)

     

        return {'FINISHED'}



#=====================================================
#     END  -   RIGIFY ZOO METARIG - HORSE   
#=====================================================
  



classes = [ OBJECT_OT_rigify_zoo_horse_metarig_ra,

          ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)



def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)




if __name__ == "__main__":
    register()







